@@ -76,6 +76,10 @@ struct PassOptions {
     ret.setDefaultOptimizationOptions();
     return ret;
   }
+
+  static PassOptions getWithoutOptimization() {
+    return PassOptions(); // defaults are to not optimize
+  }
 };
 
 //
@@ -94,9 +98,12 @@ struct PassRunner {
   PassRunner(const PassRunner&) = delete;
   PassRunner& operator=(const PassRunner&) = delete;
 
-  void setDebug(bool debug_) {
-    options.debug = debug_;
-    options.validateGlobally = debug_; // validate everything by default if debugging
+  void setDebug(bool debug) {
+    options.debug = debug;
+    options.validateGlobally = debug; // validate everything by default if debugging
+  }
+  void setDebugInfo(bool debugInfo) {
+    options.debugInfo = debugInfo;
   }
   void setValidateGlobally(bool validate) {
     options.validateGlobally = validate;
@@ -137,6 +144,9 @@ struct PassRunner {
   // Adds the default optimization passes that work on
   // entire modules as a whole, and make sense to
   // run after function passes.
+  // This is run at the very end of the optimization
+  // process - you can assume no other opts will be run
+  // afterwards.
   void addDefaultGlobalOptimizationPostPasses();
 
   // Run the passes on the module
@@ -174,7 +184,16 @@ protected:
 private:
   void doAdd(Pass* pass);
 
+  void runPass(Pass* pass);
   void runPassOnFunction(Pass* pass, Function* func);
+
+  // After running a pass, handle any changes due to
+  // how the pass is defined, such as clearing away any
+  // temporary data structures that the pass declares it
+  // invalidates.
+  // If a function is passed, we operate just on that function;
+  // otherwise, the whole module.
+  void handleAfterEffects(Pass* pass, Function* func=nullptr);
 };
 
 //
@@ -211,6 +230,11 @@ public:
   // That means that you can't rely on Walker object properties to persist across
   // your functions, and you can't expect a new object to be created for each
   // function either (which could be very inefficient).
+  //
+  // It is valid for function-parallel passes to read (but not modify) global
+  // module state, like globals or imports. However, reading other functions'
+  // contents is invalid, as function-parallel tests can be run while still
+  // adding functions to the module.
   virtual bool isFunctionParallel() { return false; }
 
   // This method is used to create instances per function for a function-parallel
@@ -218,6 +242,14 @@ public:
   // this will create the parent class.
   virtual Pass* create() { WASM_UNREACHABLE(); }
 
+  // Whether this pass modifies the Binaryen IR in the module. This is true for
+  // most passes, except for passes that have no side effects, or passes that
+  // only modify other things than Binaryen IR (for example, the Stack IR
+  // passes only modify that IR).
+  // This property is important as if Binaryen IR is modified, we need to throw
+  // out any Stack IR - it would need to be regenerated and optimized.
+  virtual bool modifiesBinaryenIR() { return true; }
+
   std::string name;
 
 protected:
@@ -263,22 +295,6 @@ public:
   }
 };
 
-// Standard passes. All passes in /passes/ are runnable from the shell,
-// but registering them here in addition allows them to communicate
-// e.g. through PassRunner::getLast
-
-// Prints out a module
-class Printer : public Pass {
-protected:
-  std::ostream& o;
-
-public:
-  Printer() : o(std::cout) {}
-  Printer(std::ostream* o) : o(*o) {}
-
-  void run(PassRunner* runner, Module* module) override;
-};
-
 } // namespace wasm
 
 #endif // wasm_pass_h
