@@ -268,7 +268,7 @@ private:
 
   void finalizeTable() {
     wasm.table.initial = wasm.table.segments[0].data.size();
-    wasm.table.max = oneIn(2) ? Address(Table::kMaxSize) : wasm.table.initial;
+    wasm.table.max = oneIn(2) ? Address(Table::kUnlimitedSize) : wasm.table.initial;
   }
 
   const Name HANG_LIMIT_GLOBAL = "hangLimit";
@@ -1438,3 +1438,6 @@ private:
 } // namespace wasm
 
 // XXX Switch class has a condition?! is it real? should the node type be the value type if it exists?!
+
+// TODO copy an existing function and replace just one node in it
+
