@@ -31,6 +31,7 @@
 #include "support/safe_integer.h"
 #include "wasm.h"
 #include "wasm-traversal.h"
+#include "ir/module-utils.h"
 
 #ifdef WASM_INTERPRETER_DEBUG
 #include "wasm-printing.h"
@@ -511,7 +512,6 @@ public:
 
   Flow visitLoop(Loop* curr) { WASM_UNREACHABLE(); }
   Flow visitCall(Call* curr) { WASM_UNREACHABLE(); }
-  Flow visitCallImport(CallImport* curr) { WASM_UNREACHABLE(); }
   Flow visitCallIndirect(CallIndirect* curr) { WASM_UNREACHABLE(); }
   Flow visitGetLocal(GetLocal *curr) { WASM_UNREACHABLE(); }
   Flow visitSetLocal(SetLocal *curr) { WASM_UNREACHABLE(); }
@@ -545,7 +545,7 @@ public:
   struct ExternalInterface {
     virtual void init(Module& wasm, SubType& instance) {}
     virtual void importGlobals(GlobalManager& globals, Module& wasm) = 0;
-    virtual Literal callImport(Import* import, LiteralList& arguments) = 0;
+    virtual Literal callImport(Function* import, LiteralList& arguments) = 0;
     virtual Literal callTable(Index index, LiteralList& arguments, Type result, SubType& instance) = 0;
     virtual void growMemory(Address oldSize, Address newSize) = 0;
     virtual void trap(const char* why) = 0;
@@ -636,9 +636,9 @@ public:
     // prepare memory
     memorySize = wasm.memory.initial;
     // generate internal (non-imported) globals
-    for (auto& global : wasm.globals) {
+    ModuleUtils::iterDefinedGlobals(wasm, [&](Global* global) {
       globals[global->name] = ConstantExpressionRunner<GlobalManager>(globals).visit(global->init).value;
-    }
+    });
     // initialize the rest of the external interface
     externalInterface->init(wasm, *self());
     // run start, if present
@@ -649,15 +649,14 @@ public:
   }
 
   // call an exported function
-  Literal callExport(Name name, LiteralList& arguments) {
+  Literal callExport(Name name, const LiteralList& arguments) {
     Export *export_ = wasm.getExportOrNull(name);
     if (!export_) externalInterface->trap("callExport not found");
     return callFunction(export_->value, arguments);
   }
 
   Literal callExport(Name name) {
-    LiteralList arguments;
-    return callExport(name, arguments);
+    return callExport(name, LiteralList());
   }
 
   // get an exported global
@@ -689,7 +688,7 @@ private:
 
 public:
   // Call a function, starting an invocation.
-  Literal callFunction(Name name, LiteralList& arguments) {
+  Literal callFunction(Name name, const LiteralList& arguments) {
     // if the last call ended in a jump up the stack, it might have left stuff for us to clean up here
     callDepth = 0;
     functionStack.clear();
@@ -697,14 +696,14 @@ public:
   }
 
   // Internal function call. Must be public so that callTable implementations can use it (refactor?)
-  Literal callFunctionInternal(Name name, LiteralList& arguments) {
+  Literal callFunctionInternal(Name name, const LiteralList& arguments) {
 
     class FunctionScope {
      public:
       std::vector<Literal> locals;
       Function* function;
 
-      FunctionScope(Function* function, LiteralList& arguments)
+      FunctionScope(Function* function, const LiteralList& arguments)
           : function(function) {
         if (function->params.size() != arguments.size()) {
           std::cerr << "Function `" << function->name << "` expects "
@@ -758,19 +757,18 @@ public:
         LiteralList arguments;
         Flow flow = generateArguments(curr->operands, arguments);
         if (flow.breaking()) return flow;
-        Flow ret = instance.callFunctionInternal(curr->target, arguments);
+        auto* func = instance.wasm.getFunction(curr->target);
+        Flow ret;
+        if (func->imported()) {
+          ret = instance.externalInterface->callImport(func, arguments);
+        } else {
+          ret = instance.callFunctionInternal(curr->target, arguments);
+        }
 #ifdef WASM_INTERPRETER_DEBUG
         std::cout << "(returned to " << scope.function->name << ")\n";
 #endif
         return ret;
       }
-      Flow visitCallImport(CallImport *curr) {
-        NOTE_ENTER("CallImport");
-        LiteralList arguments;
-        Flow flow = generateArguments(curr->operands, arguments);
-        if (flow.breaking()) return flow;
-        return instance.externalInterface->callImport(instance.wasm.getImport(curr->target), arguments);
-      }
       Flow visitCallIndirect(CallIndirect *curr) {
         NOTE_ENTER("CallIndirect");
         LiteralList arguments;
@@ -926,7 +924,6 @@ public:
       Flow visitHost(Host *curr) {
         NOTE_ENTER("Host");
         switch (curr->op) {
-          case PageSize:   return Literal((int32_t)Memory::kPageSize);
           case CurrentMemory: return Literal(int32_t(instance.memorySize));
           case GrowMemory: {
             auto fail = Literal(int32_t(-1));
@@ -942,11 +939,6 @@ public:
             instance.memorySize = newSize;
             return Literal(int32_t(ret));
           }
-          case HasFeature: {
-            Name id = curr->nameOperand;
-            if (id == WASM) return Literal(1);
-            return Literal((int32_t)0);
-          }
           default: WASM_UNREACHABLE();
         }
       }
